/**
 * @packageDocumentation
 * Ezmoney is a library that helps in manipulating monetary values.
 * It defines the interface {@link MonetaryValue} on which functions such as {@link add} and {@link format} can be called.
 */


/**
 * @public
 * Returns the provided monetary value with an amount that is positive.
 * @remarks
 * See {@link unsafeAbsolute} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value whose sign will be removed
 * @returns a `MonetaryValue` object with the same `currency` and `precision` as `monetaryValue` but whose `amount` is `abs(monetaryValue.amount)`
 */
export declare function absolute<C extends string>(monetaryValue: MonetaryValue<C>): MonetaryValue<C>;

/**
 * @public
 * Returns the sum of the two monetary values.
 * If they have a different `precision`, the highest is used.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link unsafeAdd} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to add to `monetaryValue1`; must be of the same currency
 * @returns a `MonetaryValue` object whose decimal value is the sum of the ones of `monetaryValue1` and `monetaryValue2`, with the same `currency` as `monetaryValue1` and `monetaryValue2`
 */
export declare function add<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): MonetaryValue<C>;

/**
 * @public
 * Splits a monetary value in shares of given weights, preserving the precision.
 * Useful when you want to perform successive divisions but must ensure that the sum of the quotients is equal to the provided monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * A 1-dimension Brensenham algorithm is used to handle rounding.
 * When the division of the amount by a given weight results to a non-integer number, the decimal part is reported to the next share.
 * This ensures, in a single pass over the weights, that the sum of the shares is equal to the provided monetary value.
 * It is however unpredictable as the order of the weights will change the result.
 * You may need to a implement a similar function yourself if you need control over the rounding.
 * The operation may cause of overflows or imprecisions if negative weights are provided.
 * See {@link unsafeAllocate} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value that will be split
 * @param weights - An array of numbers, positive or negative, that will determine the allocation; their sum must be different than `0`
 */
export declare function allocate<C extends string>(monetaryValue: MonetaryValue<C>, weights: number[]): MonetaryValue<C>[];

/**
 * @public
 * Returns a number whose sign represents whether the first monetary value is less than, equal to or greater than the second monetary value.
 * @remarks
 * See {@link unsafeCompare} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency as `monetaryValue1`
 * @returns respectively `-1`, `0` or `1` when `monetaryValue1` is less than, equal to or greater than `monetaryValue2`
 */
export declare function compare<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): Comparison;

/**
 * @public
 * Return values for {@link compare}.
 */
export declare const enum Comparison {
    /**
     * Less than
     */
    LT = -1,
    /**
     * Equal
     */
    EQ = 0,
    /**
     * Greater than
     */
    GT = 1
}

/**
 * @public
 * A convenience function to build a monetary value.
 * @remarks
 * See {@link unsafeCreate} for an implementation that does not validate the arguments.
 * @param amount - The amount that will be given to the monetary value; must be a safe integer
 * @param currency - The currency that will be given to the monetary value
 * @param precision - The precision that will be given to the monetary value; must a positive safe integer; defaults to `0`
 * @returns a `MonetaryValue` object whose `amount`, `currency` and `precision` are the ones provided
 */
export declare function create<C extends string>(amount: number, currency: C, precision?: number): MonetaryValue<C>;

/**
 * @public
 * Determines how to display the currency.
 */
export declare const enum CurrencyDisplay {
    /**
     * Display the currency as a localized currency symbol, such as €.
     */
    Symbol = "symbol",
    /**
     * Display the currency as the ISO code from ISO4217.
     */
    Code = "code",
    /**
     * Display the currency as a localized currency name.
     */
    Name = "name"
}

/**
 * @alpha
 * Describes how to display the negative sign.
 * Follows the [Intl.NumberFormat Unified API Proposal](https://github.com/tc39/proposal-unified-intl-numberformat)
 */
export declare const enum CurrencySign {
    /**
     * Displays the positive sign as a plus and the negative sign as a minus.
     */
    Standard = "standard",
    /**
     * Displays the positive sign as a plus and the negative sign as enclosed parentheses.
     */
    Accounting = "accounting"
}

/**
 * @public
 * Returns a monetary value whose amount is the division of the provided monetary value's amount and the decimal divider.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link unsafeDivide} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value to divide
 * @param divider - The coefficient of the number that `monetaryValue` will be divided by; must be a safe integer different than `0`
 * @param dividerPrecision - The precision of the number that `monetaryValue` will be divided by
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` whose decimal value is the division of the ones of `monetaryValue` and `divider / (10 ** dividerPrecision)`, with the same `currency` as `monetaryValue`
 */
export declare function divide<C extends string>(monetaryValue: MonetaryValue<C>, divider: number, dividerPrecision: number, roundingFunction?: RoundingFunction): MonetaryValue<C>;

/**
 * @public
 * Returns whether the two monetary values are equal.
 * @remarks
 * As with most of the other binary operators, `equal` throws when the provided monetary values have different currencies.
 * Use {@link equivalent} if you want to determine the equality of monetary values that may have a different currency.
 * See {@link unsafeEqual} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency as `monetaryValue1`
 * @returns `true` if the objects hold the same decimal value; `false` otherwise
 */
export declare function equal<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * Returns whether the two monetary values are equivalent.
 * @remarks
 * Contrary to most other binary operators, `equivalent` does not throw when the provided monetary values have different currencies.
 * Use {@link equal} if you want to determine the equality of monetary values that always have the same currency.
 * See {@link unsafeEquivalent} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` if the objects hold the same decimal value and have the same `currency`; `false` otherwise
 */
export declare function equivalent<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * Formats a monetary value into a string in the given locale.
 * @remarks
 * See {@link unsafeFormat} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value to format
 * @param locales - A string with a BCP 47 language tag, or an array of such strings.
 * @param options - An object to tweak the formatting
 * @returns a string that represents `monetaryValue` in the given `locale`
 */
export declare function format<C extends string>(monetaryValue: MonetaryValue<C>, locales?: string | string[], options?: MonetaryValueFormatOptions): string;

/**
 * @public
 * Turns a number and a currency into a monetary value.
 * @remarks
 * See {@link unsafeFromNumber} for an implementation that does not validate the arguments.
 * @param number - The number that will be used to define the amount of monetary value
 * @param currency - The currency of the monetary value
 * @param precision - The precision of the monetary value; defaults to `0`
 * @param roundingFunction - The rounding method to apply when setting the precision (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a valid `MonetaryValue` object with the provided currency and whose decimal value is the provided number, up to the provided precision
 */
export declare function fromNumber<C extends string>(number: number, currency: C, precision?: number, roundingFunction?: RoundingFunction): MonetaryValue<C>;

/**
 * @public
 * Parses a string into a monetary value.
 * The string must be the currency, followed by a space, followed by the decimal value.
 * @remarks
 * See {@link unsafeFromString} for an implementation that does not validate the return value.
 * @param str - The string to parse into a `MonetaryValue ` object; must match the regular expression `/^.+ [+-]?\d*\.?\d+$/ms`
 * @returns a valid `MonetaryValue` object
 */
export declare function fromString(str: string): MonetaryValue;

/**
 * @public
 * Returns whether the first monetary value is strictly greater than the second monetary value.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link unsafeGreaterThan} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency as `monetaryValue1`
 * @returns `true` when `monetaryValue1` holds a decimal value greater than `monetaryValue2`; `false` otherwise
 */
export declare function greaterThan<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * Returns whether the first monetary value is greater than or equal to the second monetary value.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link unsafeGreaterThanOrEqual} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency as `monetaryValue1`
 * @returns `true` when `monetaryValue1` holds a decimal value greater than or equivalent to `monetaryValue2`; `false` otherwise
 */
export declare function greaterThanOrEqual<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * Returns whether the two monetary values are identical.
 * @remarks
 * Contrary to most other binary operators, `identical` does not throw when the provided monetary values have different currencies.
 * See {@link unsafeIdentical} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` if the objects have the same `amount`, the same `currency` and the same `precision`; `false` otherwise
 */
export declare function identical<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * Returns a monetary value whose amount is the quotient of the monetary value's amount and the divider.
 * @remarks
 * The operation may cause overflows or imprecisions depending on the provided rounding function.
 * See {@link unsafeIntegerDivide} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value to divide
 * @param divider - The number that `monetaryValue` will be divided by; must be a safe integer different than `0`
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` whose decimal value is the whole quotient of the ones of `monetaryValue` and `divider`, with the same `currency` as `monetaryValue`
 */
export declare function integerDivide<C extends string>(monetaryValue: MonetaryValue<C>, divider: number, roundingFunction?: RoundingFunction): MonetaryValue<C>;

/**
 * @public
 * Returns whether {@link format} and {@link unsafeFormat} are supported in your environment.
 * @remarks
 * Implementation taken from [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/toLocaleString#Checking_for_support_for_locales_and_options_arguments).
 */
export declare function isFormatSupported(): boolean;

/**
 * @public
 * Returns whether the provided argument is a monetary value.
 * Validates both the shape of the object and whether it is a valid monetary value.
 * Can be used as a type guard.
 * @remarks
 * Must be chained to another type guard if you want a more specific type for currency.
 * @param arg - Value to check
 * @returns `true` if `arg` is a of type `MonetaryValue` and has a valid `amount` and a valid `precision`; `false` otherwise
 */
export declare function isMonetaryValue(arg: unknown): arg is MonetaryValue;

/**
 * @public
 * Returns whether the first monetary value is strictly less than the second monetary value.
 * @remarks
 * See {@link unsafeLessThan} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency as `monetaryValue1`
 * @returns `true` when `monetaryValue1` holds a decimal value strictly less than `monetaryValue2`; `false` otherwise
 */
export declare function lessThan<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * Returns whether the first monetary value is less than or equal to the second monetary value.
 * @remarks
 * See {@link unsafeLessThanOrEqual} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency as `monetaryValue1`
 * @returns `true` when `monetaryValue1` holds a decimal value less than or equivalent to `monetaryValue2`; `false` otherwise
 */
export declare function lessThanOrEqual<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * The locale matching algorithms.
 */
export declare const enum LocaleMatcher {
    /**
     * Locale is selected by the runtime.
     */
    BestFit = "best fit",
    /**
     * Locale is selected according to the Lookup algorithm in [BCP 47](https://tools.ietf.org/html/rfc4647#section-3.4)
     */
    Lookup = "lookup"
}

/**
 * @public
 * Returns an equivalent monetary value, whose precision is the provided one if it is equal or higher.
 * @remarks
 * The operation may cause overflows or imprecisions if the provided precision is big enough.
 * See {@link unsafeMatchPrecision} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value whose precision may be changed
 * @param precision - The precision that the monetary value shall match; must be a positive safe integer
 * @returns a `MonetaryValue` object equivalent to `monetaryValue` and whose `precision` may be equal to `precision`
 */
export declare function matchPrecision<C extends string>(monetaryValue: MonetaryValue<C>, precision: number): MonetaryValue<C>;

/**
 * @public
 * Returns the monetary value with the greatest decimal amount.
 * @remarks
 * See {@link unsafeMaximum} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency
 * @returns `monetaryValue1` if its decimal value is greater than the one of `monetaryValue2` or, when they are equal, if its precision is lower than the one of `monetaryValue2`; otherwise, `monetaryValue2`
 */
export declare function maximum<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): MonetaryValue<C>;

/**
 * @public
 * Returns the monetary value with the smallest decimal amount.
 * @remarks
 * See {@link unsafeMinimum} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency
 * @returns `monetaryValue2` if its decimal value is greater than the one of `monetaryValue1` or, when they are equal, if its precision is lower than the one of `monetaryValue1`; otherwise, `monetaryValue1`
 */
export declare function minimum<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): MonetaryValue<C>;

/**
 * @public
 * A decimal value with a currency.
 */
export declare interface MonetaryValue<C extends string = string> {
    /**
     * The coefficient part of the decimal value.
     */
    readonly amount: number;
    /**
     * A string representation of the currency.
     * Currencies are deemed equal if and only if they have the same string representation.
     */
    readonly currency: C;
    /**
     * The "exponent" part of the decimal value.
     * This is number of decimal places that the monetary value must have.
     */
    readonly precision: number;
}

/**
 * @public
 * The options that can be passed to {@link format} or {@link unsafeFormat} to tweak to formatting.
 */
export declare interface MonetaryValueFormatOptions {
    /**
     * The locale matching algorithm to use.
     * For more information, see [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl#Locale_negotiation).
     */
    localeMatcher?: LocaleMatcher;
    /**
     * Determines how to display the currency part of the string.
     */
    currencyDisplay?: CurrencyDisplay;
    /**
     * Whether to use grouping separators, such as thousands separators or thousand/lakh/crore separators.
     */
    useGrouping?: boolean;
    /**
     * @alpha
     * Determines in which situation the sign should be displayed.
     * Follows the [Intl.NumberFormat Unified API Proposal](https://github.com/tc39/proposal-unified-intl-numberformat).
     */
    signDisplay?: SignDisplay;
    /**
     * @alpha
     * Determines how to display the negative sign.
     * Follows the [Intl.NumberFormat Unified API Proposal](https://github.com/tc39/proposal-unified-intl-numberformat)
     */
    currencySign?: CurrencySign;
}

/**
 * @public
 * Returns a monetary value whose amount is the product of the monetary value's amount and the decimal factor.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link unsafeMultiply} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value to multiply
 * @param factor - The coefficient of the number that `monetaryValue` will be multiplied by; must be a safe integer
 * @param factorPrecision - The precision of the number that `monetaryValue` will be multiplied by
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half-to-even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` whose decimal value is the product of the ones of `monetaryValue` and `factor / (10 ** factorPrecision)`, with the same `currency` as `monetaryValue`
 */
export declare function multiply<C extends string>(monetaryValue: MonetaryValue<C>, factor: number, factorPrecision: number, roundingFunction?: RoundingFunction): MonetaryValue<C>;

/**
 * @public
 * Returns the provided monetary value with an amount of the opposite sign.
 * @remarks
 * See {@link unsafeNegate} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value whose sign will be changed
 * @returns a `MonetaryValue` object with the same `currency` and `precision` as `monetaryValue` but whose `amount` is `-monetaryValue.amount`
 */
export declare function negate<C extends string>(monetaryValue: MonetaryValue<C>): MonetaryValue<C>;

/**
 * @public
 * Rounds to the integer that is the farthest from zero.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division; _Unused_
 */
export declare function roundAwayFromZero(wholePart: number, numeratorFractionalPart: number, denominatorFractionalPart?: number): number;

/**
 * @public
 * Rounds to the smallest integer
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division; _Unused_
 */
export declare function roundDown(wholePart: number, numeratorFractionalPart: number, denominatorFractionalPart?: number): number;

/**
 * @public
 * Rounds to the closest integer, and away zero if the number is an exact half.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division
 */
export declare function roundHalfAwayFromZero(wholePart: number, numeratorFractionalPart: number, denominatorFractionalPart: number): number;

/**
 * @public
 * Rounds to the closest integer, and down if the number is an exact half.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division
 */
export declare function roundHalfDown(wholePart: number, numeratorFractionalPart: number, denominatorFractionalPart: number): number;

/**
 * @public
 * Rounds to the closest integer, and to the closest even integer if the number is an exact half.
 * The most common rounding for monetary value as the tie-breaking rule is not biased towards a particular direction.
 * Also called the Banker's rounding.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division
 */
export declare function roundHalfToEven(wholePart: number, numeratorFractionalPart: number, denominatorFractionalPart: number): number;

/**
 * @public
 * Rounds to the closest integer, and to the closest odd integer if the number is an exact half.
 * The tie-breaking rule is not biased towards a particular direction.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division
 */
export declare function roundHalfToOdd(wholePart: number, numeratorFractionalPart: number, denominatorFractionalPart: number): number;

/**
 * @public
 * Rounds to the closest integer, and towards zero if the number is an exact half.
 * A very common rounding for regular numbers.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division
 */
export declare function roundHalfTowardsZero(wholePart: number, numeratorFractionalPart: number, denominatorFractionalPart: number): number;

/**
 * @public
 * Rounds to the closest integer, and up if the number is an exact half.
 * The most common rounding for regular numbers.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division
 */
export declare function roundHalfUp(wholePart: number, numeratorFractionalPart: number, denominatorFractionalPart: number): number;

/**
 * @public
 * A rounding function returns an integer from a potentially non-integer value.
 * Such a value is represented as the sum of an integer and a fraction.
 * The fraction is always between -1 and 1, non included, and its denominator is the original divider.
 * Rounding functions are not meant to be used as-is;
 * they are to be passed to functions that may perform rounding, such as {@link divide}.
 * As such, none of them validate their arguments.
 * When a function is expected to perform rounding, it uses {@link roundHalfToEven} by default.
 * This can be overridden by passing a rounding function in the last argument.
 * Most of the common rounding functions are defined and exported in Ezmoney,
 * but you can of course implement your own.
 */
export declare type RoundingFunction = (
/**
 * The whole part of the number to be rounded
 */
wholePart: number, 
/**
 * The numerator of the fractional part of the number to be rounded
 */
numeratorFractionalPart: number, 
/**
 * The denominator of the fractional part of the number to be rounded
 */
denominatorFractionalPart: number) => number;

/**
 * @public
 * Rounds to the integer that is the closest to zero.
 * This is equivalent to truncating to the whole part.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division; _Unused_
 * @param denominatorFractionalPart - The divider of the prior division; _Unused_
 */
export declare function roundTowardsZero(wholePart: number, numeratorFractionalPart?: number, denominatorFractionalPart?: number): number;

/**
 * @public
 * Rounds to the greatest integer
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division; _Unused_
 */
export declare function roundUp(wholePart: number, numeratorFractionalPart: number, denominatorFractionalPart?: number): number;

/**
 * @public
 * Returns a monetary value whose precision is the provided one.
 * It is equivalent to the provided monetary value if the precision is equal or higher than the one of the provided monetary value.
 * Otherwise, it rounds the amount.
 * @remarks
 * The operation may cause overflows or imprecisions if the provided precision is big enough.
 * Also, note that the operation may be destructive if the precision is lower than the one of the provided monetary value.
 * See {@link unsafeSetPrecision} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value whose precision will be changed
 * @param precision - The precision that the monetary value shall be set to; must be a positive safe integer
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` object whose `precision` is equal to `precision` and that may be equivalent to `monetaryValue`
 */
export declare function setPrecision<C extends string>(monetaryValue: MonetaryValue<C>, precision: number, roundingFunction?: RoundingFunction): MonetaryValue<C>;

/**
 * @alpha
 * Describes in which situation the sign should be displayed.
 * Follows the [Intl.NumberFormat Unified API Proposal](https://github.com/tc39/proposal-unified-intl-numberformat).
 */
export declare const enum SignDisplay {
    /**
     * Display the sign only when the amount is negative.
     */
    Auto = "auto",
    /**
     * Always display the sign.
     */
    Always = "always",
    /**
     * Never display the sign.
     */
    Never = "never",
    /**
     * Always display the sign, except when the amount is zero.
     */
    ExceptZero = "except-zero"
}

/**
 * @public
 * Returns the difference of the two monetary values.
 * If they have a different `precision`, the highest is used.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link unsafeSubtract} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to subtract from `monetaryValue1`; must be of the same currency
 * @returns a `MonetaryValue` whose decimal value is the difference between the ones of `monetaryValue1` and `monetaryValue2`, with the same `currency` as `monetaryValue1` and `monetaryValue2`
 */
export declare function subtract<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): MonetaryValue<C>;

/**
 * @public
 * Returns the decimal value of the provided monetary value.
 * @remarks
 * See {@link unsafeToNumber} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value to coerce to a number
 */
export declare function toNumber<C extends string>(monetaryValue: MonetaryValue<C>): number;

/**
 * @public
 * Turns a monetary value into a predictable string.
 * It is formed of the currency, followed by a space, followed by the decimal value.
 * @remarks
 * See {@link unsafeToString} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value to turn into a `string`
 * @returns a predictable string representation of the provided `MonetaryValue` object
 */
export declare function toString(monetaryValue: MonetaryValue): string;

/**
 * @public
 * Returns the provided monetary value with an amount that is positive.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link absolute} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value whose sign will be removed
 * @returns a `MonetaryValue` object with the same `currency` and `precision` as `monetaryValue` but whose `amount` is `abs(monetaryValue.amount)`
 */
export declare function unsafeAbsolute<C extends string>(monetaryValue: MonetaryValue<C>): MonetaryValue<C>;

/**
 * @public
 * Returns the sum of the two monetary values.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link add} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to add to `monetaryValue1`
 * @returns a `MonetaryValue` object whose decimal value is the sum of the ones of `monetaryValue1` and `monetaryValue2`, with the same `currency` as `monetaryValue1` and `monetaryValue2`
 */
export declare function unsafeAdd<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): MonetaryValue<C>;

/**
 * @public
 * Splits a monetary value in shares of given weights, preserving the precision.
 * Useful when you want to perform successive divisions but must ensure that the sum of the quotients is equal to the provided monetary value.
 * @remarks
 * A 1-dimension Brensenham algorithm is used to handle rounding.
 * When the division of the amount by a given weight results to a non-integer number, the decimal part is reported to the next share.
 * This ensures, in a single pass over the weights, that the sum of the shares is equal to the provided monetary value.
 * It is however unpredictable as the order of the weights will change the result.
 * You may need to a implement a similar function yourself if you need control over the rounding.
 * The operation may cause of overflows or imprecisions if negative weights are provided.
 * See {@link allocate} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value that will be split
 * @param weights - An array of numbers, positive or negative, that will determine the allocation
 */
export declare function unsafeAllocate<C extends string>(monetaryValue: MonetaryValue<C>, weights: number[]): MonetaryValue<C>[];

/**
 * @public
 * Returns a number whose sign represents whether the first monetary value is less than, equal to or greater than the second monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link compare} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns respectively `-1`, `0` or `1` when `monetaryValue1` is less than, equal to or greater than `monetaryValue2`
 */
export declare function unsafeCompare<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): Comparison;

/**
 * @public
 * A convenience function to build a monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link create} for an implementation that validates the arguments.
 * @param amount - The amount that will be given to the monetary value
 * @param currency - The currency that will be given to the monetary value
 * @param precision - The precision that will be given to the monetary value; defaults to `0`
 * @returns a `MonetaryValue` object whose `amount`, `currency` and `precision` are the ones provided
 */
export declare function unsafeCreate<C extends string>(amount: number, currency: C, precision?: number): MonetaryValue<C>;

/**
 * @public
 * Returns a monetary value whose amount is the division of the provided monetary value's amount and the decimal divider.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link divide} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value to divide
 * @param divider - The coefficient of the number that `monetaryValue` will be divided by
 * @param dividerPrecision - The precision of the number that `monetaryValue` will be divided by
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` whose decimal value is the division of the ones of `monetaryValue` and `divider / (10 ** dividerPrecision)`, with the same `currency` as `monetaryValue`
 */
export declare function unsafeDivide<C extends string>(monetaryValue: MonetaryValue<C>, divider: number, dividerPrecision: number, roundingFunction?: RoundingFunction): MonetaryValue<C>;

/**
 * @public
 * Returns whether the two monetary values are equal.
 * Does not check the validity of the arguments.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link equal} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` if the objects hold the same decimal value; `false` otherwise
 */
export declare function unsafeEqual<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * Returns whether the two monetary values are equivalent.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link equivalent} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` if the objects hold the same decimal value and have the same `currency`; `false` otherwise
 */
export declare function unsafeEquivalent<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * Formats a monetary value into a string in the given locale.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link format} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value to format
 * @param locales - A string with a BCP 47 language tag, or an array of such strings
 * @param options - An object to tweak the formatting
 * @returns a string that represents `monetaryValue` in the given `locale`
 */
export declare function unsafeFormat<C extends string>(monetaryValue: MonetaryValue<C>, locales?: string | string[], formatOptions?: MonetaryValueFormatOptions): string;

/**
 * @public
 * Turns a number and a currency into a monetary value.
 * Does not check the validity of the input.
 * @remarks
 * See {@link fromNumber} for an implementation that validates the arguments.
 * @param number - The number that will be used to define the amount of monetary value
 * @param currency - The currency of the monetary value
 * @param precision - The precision of the monetary value; defaults to `0`
 * @param roundingFunction - The rounding method to apply when setting the precision (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` object with the provided currency and whose decimal value is the provided number, up to the provided precision
 */
export declare function unsafeFromNumber<C extends string>(number: number, currency: C, precision?: number, roundingFunction?: RoundingFunction): MonetaryValue<C>;

/**
 * @public
 * Parses a string into a monetary value.
 * The string must be the currency, followed by a space, followed by the decimal value.
 * Does not check the validity of the output.
 * @remarks
 * See {@link fromString} for an implementation that validates the return value.
 * @param str - The string to parse into a `MonetaryValue ` object; must match the regular expression `/^.+ [+-]?\d*\.?\d+$/ms`
 * @returns a `MonetaryValue` object
 */
export declare function unsafeFromString(str: string): MonetaryValue;

/**
 * @public
 * Returns whether the first monetary value is strictly greater than the second monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link greaterThan} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` when `monetaryValue1` holds a decimal value greater than `monetaryValue2`; `false` otherwise
 */
export declare function unsafeGreaterThan<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * Returns whether the first monetary value is greater than or equal to the second monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link greaterThanOrEqual} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` when `monetaryValue1` holds a decimal value greater than or equivalent to `monetaryValue2`; `false` otherwise
 */
export declare function unsafeGreaterThanOrEqual<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * Returns whether the two monetary values are identical.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link identical} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` if the objects have the same `amount`, the same `currency` and the same `precision`; `false` otherwise
 */
export declare function unsafeIdentical<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * Returns a monetary value whose amount is the quotient of the monetary value's amount and the divider.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions depending on the provided rounding function.
 * See {@link integerDivide} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value to divide
 * @param divider - The number that `monetaryValue` will be divided by
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` whose decimal value is the whole quotient of the ones of `monetaryValue` and `divider`, with the same `currency` as `monetaryValue`
 */
export declare function unsafeIntegerDivide<C extends string>(monetaryValue: MonetaryValue<C>, divider: number, roundingFunction?: RoundingFunction): MonetaryValue<C>;

/**
 * @public
 * Returns whether the first monetary value is strictly less than the second monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link lessThan} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` when `monetaryValue1` holds a decimal value strictly less than `monetaryValue2`; `false` otherwise
 */
export declare function unsafeLessThan<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * Returns whether the first monetary value is less than or equal to the second monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link lessThanOrEqual} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` when `monetaryValue1` holds a decimal value less than or equivalent to `monetaryValue2`; `false` otherwise
 */
export declare function unsafeLessThanOrEqual<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): boolean;

/**
 * @public
 * Returns an equivalent monetary value, whose precision is the provided one if it is equal or higher.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions if the provided precision is big enough.
 * See {@link matchPrecision} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value whose precision may be changed
 * @param precision - The precision that the monetary value shall match
 * @returns a `MonetaryValue` object equivalent to `monetaryValue` and whose `precision` may be equal to `precision`
 */
export declare function unsafeMatchPrecision<C extends string>(monetaryValue: MonetaryValue<C>, precision: number): MonetaryValue<C>;

/**
 * @public
 * Returns the monetary value with the greatest decimal amount.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link maximum} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`
 * @returns `monetaryValue1` if its decimal value is greater than the one of `monetaryValue2` or, when they are equal, if its precision is lower than the one of `monetaryValue2`; otherwise, `monetaryValue2`
 */
export declare function unsafeMaximum<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): MonetaryValue<C>;

/**
 * @public
 * Returns the monetary value with the smallest decimal amount.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link minimum} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`
 * @returns `monetaryValue2` if its decimal value is greater than the one of `monetaryValue1` or, when they are equal, if its precision is lower than the one of `monetaryValue1`; otherwise, `monetaryValue1`
 */
export declare function unsafeMinimum<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): MonetaryValue<C>;

/**
 * @public
 * Returns a monetary value whose amount is the product of the monetary value's amount and the decimal factor.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link multiply} for an implementation that validates the arguments.
 * @param factor - The coefficient of the number that `monetaryValue` will be multiplied by
 * @param factorPrecision - The precision of the number that `monetaryValue` will be multiplied by
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half-to-even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` whose decimal value is the product of the ones of `monetaryValue` and `factor / (10 ** factorPrecision)`, with the same `currency` as `monetaryValue`
 */
export declare function unsafeMultiply<C extends string>(monetaryValue: MonetaryValue<C>, factor: number, factorPrecision: number, roundingFunction?: RoundingFunction): MonetaryValue<C>;

/**
 * @public
 * Returns the provided monetary value with an amount of the opposite sign.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link negate} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value whose sign will be changed
 * @returns a `MonetaryValue` object with the same `currency` and `precision` as `monetaryValue` but whose `amount` is `-monetaryValue.amount`
 */
export declare function unsafeNegate<C extends string>(monetaryValue: MonetaryValue<C>): MonetaryValue<C>;

/**
 * @public
 * Returns a monetary value whose precision is the provided one.
 * It is equivalent to the provided monetary value if the precision is equal or higher than the one of the provided monetary value.
 * Otherwise, it rounds the amount.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions if the provided precision is big enough.
 * Also, note that the operation may be destructive if the precision is lower than the one of the provided monetary value.
 * See {@link setPrecision} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value whose precision will be changed
 * @param precision - The precision that the monetary value shall be set to
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` object whose `precision` is equal to `precision` and that may be equivalent to `monetaryValue`
 */
export declare function unsafeSetPrecision<C extends string>(monetaryValue: MonetaryValue<C>, precision: number, roundingFunction?: RoundingFunction): MonetaryValue<C>;

/**
 * @public
 * Returns the difference of the two monetary values.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link subtract} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to subtract to the first
 */
export declare function unsafeSubtract<C extends string>(monetaryValue1: MonetaryValue<C>, monetaryValue2: MonetaryValue<C>): MonetaryValue<C>;

/**
 * @public
 * @param monetaryValue - The monetary value to coerce to a number
 */
export declare function unsafeToNumber<C extends string>(monetaryValue: MonetaryValue<C>): number;

/**
 * @public
 * Turns a monetary value into a predictable string.
 * It is formed of the currency, followed by a space, followed by the decimal value.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link toString} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value to turn into a `string`
 * @returns a predictable string representation of the provided `MonetaryValue` object
 */
export declare function unsafeToString<C extends string>(monetaryValue: MonetaryValue<C>): string;

export { }
