/**
 * @internal
 * Computes the minimum of two numbers.
 * @param a - First number
 * @param b - Second number
 * @returns the smallest number between `a` and `b`
 */
function min(a, b) {
    return Math.min(a, b);
}
/**
 * @internal
 * Computes the maximum of two numbers.
 * @param a - First number
 * @param b - Second number
 * @returns the largest number between `a` and `b`
 */
function max(a, b) {
    return Math.max(a, b);
}
/**
 * @internal
 * Computes the sign of a number.
 * @param n - A number
 * @returns `1` if `n` is positive or null; `-1` otherwise
 */
function sign(n) {
    return Object.is(Math.abs(n), n) ? 1 : -1;
}
/**
 * @internal
 * Computes the absolute value of a number.
 * @param n - A number
 * @returns the absolute value of `n`
 */
function abs(n) {
    return Math.abs(n);
}
/**
 * @internal
 * Computes the sum of an array of numbers.
 * @param ns - An array of numbers
 * @returns the sum of all the numbers in `ns`; `0` if `ns` is empty
 */
function sum(ns) {
    var length = ns.length;
    var total = 0;
    for (var i = 0; i < length; i++) {
        total += ns[i];
    }
    return total;
}
/**
 * @internal
 * Raises 10 to the power of the provided number.
 * @param e - The exponent
 * @returns `10 ** e`
 */
function pow10(e) {
    return Math.pow(10, e);
}
/**
 * @internal
 * Splits a number in whole shares of given weights.
 * @remarks
 * A 1-dimension Brensenham algorithm is used to handle rounding: shares are rounded down but the decimal part contributes to an incremental error that is added to the shares as they are computed.
 * Negative and non-integer weights are supported, as long as their sum is a strictly positive safe integer.
 * The operation may cause overflows or imprecisions if some weights are negative and the provided amount is great enough.
 * @param n - The number to split
 * @param weights - The weights according to which `n` should be split
 * @param total - The sum of the weights; must be a strictly positive safe integer
 * @returns an array of integers of the same length as `weights` and whose sum is equal to `n`
 */
function allocate$1(n, weights, total) {
    var length = weights.length;
    var shares = new Array(length);
    var crumbs = 0;
    var sum = 0;
    var i = 0;
    while (i < length - 1) {
        var share = (weights[i] / total) * n + crumbs;
        var wholeShare = Math.floor(share);
        crumbs = share - wholeShare;
        sum += wholeShare;
        shares[i] = wholeShare;
        ++i;
    }
    shares[i] = n - sum;
    return shares;
}

function assertSafeInteger(i) {
    if (!Number.isSafeInteger(i)) {
        throw new RangeError('Number is not a safe integer');
    }
}
function assertNonZeroSafeInteger(i) {
    if (!(i !== 0 && Number.isSafeInteger(i))) {
        throw new RangeError('Number is not a safe integer different than zero');
    }
}
function assertStrictlyPositiveSafeInteger(i) {
    if (!(i > 0 && Number.isSafeInteger(i))) {
        throw new RangeError('Number is not a strictly positive safe integer');
    }
}
function assertValidPrecision(i) {
    if (!(i >= 0 && i <= 15 && Number.isInteger(i))) {
        throw new RangeError('Number is not valid precision as it is not an integer between 0 and 15');
    }
}
function assertValidMonetaryValue(monetaryValue) {
    var amount = monetaryValue.amount, precision = monetaryValue.precision;
    if (!(precision >= 0 && precision <= 15 && Number.isInteger(precision))) {
        throw new RangeError('Precision is not valid as it is not an integer between 0 and 15');
    }
    if (!Number.isSafeInteger(amount)) {
        throw new RangeError('Amount is not valid as it is not a safe integer');
    }
}
function assertSameCurrency(monetaryValue1, monetaryValue2) {
    if (monetaryValue1.currency !== monetaryValue2.currency) {
        throw new TypeError('MonetaryValue objects do not have the same currency');
    }
}

/**
 * @public
 * Rounds to the smallest integer
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division; _Unused_
 */
function roundDown(wholePart, numeratorFractionalPart, 
// eslint-disable-next-line @typescript-eslint/no-unused-vars
denominatorFractionalPart) {
    return numeratorFractionalPart === 0
        ? wholePart
        : wholePart + min(0, sign(numeratorFractionalPart));
}
/**
 * @public
 * Rounds to the greatest integer
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division; _Unused_
 */
function roundUp(wholePart, numeratorFractionalPart, 
// eslint-disable-next-line @typescript-eslint/no-unused-vars
denominatorFractionalPart) {
    return numeratorFractionalPart === 0
        ? wholePart
        : wholePart + max(0, sign(numeratorFractionalPart));
}
/**
 * @public
 * Rounds to the integer that is the closest to zero.
 * This is equivalent to truncating to the whole part.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division; _Unused_
 * @param denominatorFractionalPart - The divider of the prior division; _Unused_
 */
function roundTowardsZero(wholePart, 
// eslint-disable-next-line @typescript-eslint/no-unused-vars
numeratorFractionalPart, 
// eslint-disable-next-line @typescript-eslint/no-unused-vars
denominatorFractionalPart) {
    return wholePart;
}
/**
 * @public
 * Rounds to the integer that is the farthest from zero.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division; _Unused_
 */
function roundAwayFromZero(wholePart, numeratorFractionalPart, 
// eslint-disable-next-line @typescript-eslint/no-unused-vars
denominatorFractionalPart) {
    return numeratorFractionalPart === 0
        ? wholePart
        : wholePart + sign(numeratorFractionalPart);
}
/**
 * @public
 * Rounds to the closest integer, and up if the number is an exact half.
 * The most common rounding for regular numbers.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division
 */
function roundHalfUp(wholePart, numeratorFractionalPart, denominatorFractionalPart) {
    var doubleN = 2 * abs(numeratorFractionalPart);
    if (doubleN > denominatorFractionalPart) {
        return roundAwayFromZero(wholePart, numeratorFractionalPart);
    }
    if (doubleN < denominatorFractionalPart) {
        return roundTowardsZero(wholePart);
    }
    return roundUp(wholePart, numeratorFractionalPart);
}
/**
 * @public
 * Rounds to the closest integer, and down if the number is an exact half.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division
 */
function roundHalfDown(wholePart, numeratorFractionalPart, denominatorFractionalPart) {
    var doubleN = 2 * abs(numeratorFractionalPart);
    if (doubleN > denominatorFractionalPart) {
        return roundAwayFromZero(wholePart, numeratorFractionalPart);
    }
    if (doubleN < denominatorFractionalPart) {
        return roundTowardsZero(wholePart);
    }
    return roundDown(wholePart, numeratorFractionalPart);
}
/**
 * @public
 * Rounds to the closest integer, and towards zero if the number is an exact half.
 * A very common rounding for regular numbers.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division
 */
function roundHalfTowardsZero(wholePart, numeratorFractionalPart, denominatorFractionalPart) {
    if (2 * abs(numeratorFractionalPart) > denominatorFractionalPart) {
        return roundAwayFromZero(wholePart, numeratorFractionalPart);
    }
    return roundTowardsZero(wholePart);
}
/**
 * @public
 * Rounds to the closest integer, and away zero if the number is an exact half.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division
 */
function roundHalfAwayFromZero(wholePart, numeratorFractionalPart, denominatorFractionalPart) {
    if (2 * abs(numeratorFractionalPart) >= denominatorFractionalPart) {
        return roundAwayFromZero(wholePart, numeratorFractionalPart);
    }
    return roundTowardsZero(wholePart);
}
/**
 * @public
 * Rounds to the closest integer, and to the closest even integer if the number is an exact half.
 * The most common rounding for monetary value as the tie-breaking rule is not biased towards a particular direction.
 * Also called the Banker's rounding.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division
 */
function roundHalfToEven(wholePart, numeratorFractionalPart, denominatorFractionalPart) {
    if (2 * abs(numeratorFractionalPart) === denominatorFractionalPart) {
        if (wholePart % 2 === 0) {
            return wholePart;
        }
        return wholePart + sign(numeratorFractionalPart);
    }
    return roundHalfUp(wholePart, numeratorFractionalPart, denominatorFractionalPart);
}
/**
 * @public
 * Rounds to the closest integer, and to the closest odd integer if the number is an exact half.
 * The tie-breaking rule is not biased towards a particular direction.
 * @param wholePart - The whole part of the number to round
 * @param numeratorFractionalPart - The remainder of the prior division
 * @param denominatorFractionalPart - The divider of the prior division
 */
function roundHalfToOdd(wholePart, numeratorFractionalPart, denominatorFractionalPart) {
    if (2 * abs(numeratorFractionalPart) === denominatorFractionalPart) {
        if (wholePart % 2 === 0) {
            return wholePart + sign(numeratorFractionalPart);
        }
        return wholePart;
    }
    return roundHalfUp(wholePart, numeratorFractionalPart, denominatorFractionalPart);
}

/**
 * @public
 * Turns a number and a currency into a monetary value.
 * @remarks
 * See {@link unsafeFromNumber} for an implementation that does not validate the arguments.
 * @param number - The number that will be used to define the amount of monetary value
 * @param currency - The currency of the monetary value
 * @param precision - The precision of the monetary value; defaults to `0`
 * @param roundingFunction - The rounding method to apply when setting the precision (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a valid `MonetaryValue` object with the provided currency and whose decimal value is the provided number, up to the provided precision
 */
function fromNumber(number, currency, precision, roundingFunction) {
    if (precision === void 0) { precision = 0; }
    if (roundingFunction === void 0) { roundingFunction = roundHalfToEven; }
    // Does not use unsafeFromNumber because we need to validate mid-computation
    assertValidPrecision(precision);
    var scaled = number * pow10(precision);
    var wholePart = Math.floor(scaled);
    assertSafeInteger(wholePart);
    var fractionalPart = scaled - wholePart;
    var amount = roundingFunction(wholePart, fractionalPart * 100, 100);
    return { amount: amount, currency: currency, precision: precision };
}
/**
 * @public
 * Turns a number and a currency into a monetary value.
 * Does not check the validity of the input.
 * @remarks
 * See {@link fromNumber} for an implementation that validates the arguments.
 * @param number - The number that will be used to define the amount of monetary value
 * @param currency - The currency of the monetary value
 * @param precision - The precision of the monetary value; defaults to `0`
 * @param roundingFunction - The rounding method to apply when setting the precision (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` object with the provided currency and whose decimal value is the provided number, up to the provided precision
 */
function unsafeFromNumber(number, currency, precision, roundingFunction) {
    if (precision === void 0) { precision = 0; }
    if (roundingFunction === void 0) { roundingFunction = roundHalfToEven; }
    var scaled = number * pow10(precision);
    var wholePart = Math.floor(scaled);
    var fractionalPart = scaled - wholePart;
    var amount = roundingFunction(wholePart, fractionalPart * 100, 100);
    return { amount: amount, currency: currency, precision: precision };
}
/**
 * @public
 * Returns the decimal value of the provided monetary value.
 * @remarks
 * See {@link unsafeToNumber} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value to coerce to a number
 */
function toNumber(monetaryValue) {
    assertValidMonetaryValue(monetaryValue);
    return unsafeToNumber(monetaryValue);
}
/**
 * @public
 * @param monetaryValue - The monetary value to coerce to a number
 */
function unsafeToNumber(monetaryValue) {
    return monetaryValue.amount / pow10(monetaryValue.precision);
}
/**
 * @public
 * Parses a string into a monetary value.
 * The string must be the currency, followed by a space, followed by the decimal value.
 * @remarks
 * See {@link unsafeFromString} for an implementation that does not validate the return value.
 * @param str - The string to parse into a `MonetaryValue ` object; must match the regular expression `/^.+ [+-]?\d*\.?\d+$/ms`
 * @returns a valid `MonetaryValue` object
 */
function fromString(str) {
    var monetaryValue = unsafeFromString(str);
    assertValidMonetaryValue(monetaryValue);
    return monetaryValue;
}
/**
 * @public
 * Parses a string into a monetary value.
 * The string must be the currency, followed by a space, followed by the decimal value.
 * Does not check the validity of the output.
 * @remarks
 * See {@link fromString} for an implementation that validates the return value.
 * @param str - The string to parse into a `MonetaryValue ` object; must match the regular expression `/^.+ [+-]?\d*\.?\d+$/ms`
 * @returns a `MonetaryValue` object
 */
function unsafeFromString(str) {
    var result = fromStringParsingRegex.exec(str);
    if (!result) {
        throw new SyntaxError('Cannot parse monetary value');
    }
    var currency = result[1], sign = result[2], wholePartStr = result[3], decimalPoint = result[4], decimalPartStr = result[5];
    var precision = decimalPoint ? decimalPartStr.length : 0;
    // When there is no decimal point, there is no decimal part either
    // The last group in the regex captures the last digit of the whole part
    var amount = parseInt(wholePartStr + decimalPartStr, 10);
    if (sign === '-') {
        amount *= -1;
    }
    return { amount: amount, currency: currency, precision: precision };
}
/**
 * @internal
 */
var fromStringParsingRegex = /^([\s\S]+) ([+-])?(\d*)(\.?)(\d+)$/m;
/**
 * @public
 * Turns a monetary value into a predictable string.
 * It is formed of the currency, followed by a space, followed by the decimal value.
 * @remarks
 * See {@link unsafeToString} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value to turn into a `string`
 * @returns a predictable string representation of the provided `MonetaryValue` object
 */
function toString(monetaryValue) {
    assertValidMonetaryValue(monetaryValue);
    return unsafeToString(monetaryValue);
}
/**
 * @public
 * Turns a monetary value into a predictable string.
 * It is formed of the currency, followed by a space, followed by the decimal value.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link toString} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value to turn into a `string`
 * @returns a predictable string representation of the provided `MonetaryValue` object
 */
function unsafeToString(monetaryValue) {
    var amount = monetaryValue.amount, precision = monetaryValue.precision, currency = monetaryValue.currency;
    var signStr = amount < 0 ? '-' : '';
    var absoluteAmountStr = abs(amount).toString();
    if (precision === 0) {
        return currency + " " + signStr + absoluteAmountStr;
    }
    if (absoluteAmountStr.length <= precision) {
        var leadingZeros = new Array(precision - absoluteAmountStr.length)
            .fill(0)
            .join('');
        return currency + " " + signStr + "0." + leadingZeros + absoluteAmountStr;
    }
    var splitIndex = absoluteAmountStr.length - precision;
    var wholePartStr = absoluteAmountStr.substring(0, splitIndex);
    var decimalPartStr = absoluteAmountStr.substring(splitIndex);
    return currency + " " + signStr + wholePartStr + "." + decimalPartStr;
}

/**
 * @public
 * Returns an equivalent monetary value, whose precision is the provided one if it is equal or higher.
 * @remarks
 * The operation may cause overflows or imprecisions if the provided precision is big enough.
 * See {@link unsafeMatchPrecision} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value whose precision may be changed
 * @param precision - The precision that the monetary value shall match; must be a positive safe integer
 * @returns a `MonetaryValue` object equivalent to `monetaryValue` and whose `precision` may be equal to `precision`
 */
function matchPrecision(monetaryValue, precision) {
    assertValidMonetaryValue(monetaryValue);
    assertValidPrecision(precision);
    return unsafeMatchPrecision(monetaryValue, precision);
}
/**
 * @public
 * Returns an equivalent monetary value, whose precision is the provided one if it is equal or higher.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions if the provided precision is big enough.
 * See {@link matchPrecision} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value whose precision may be changed
 * @param precision - The precision that the monetary value shall match
 * @returns a `MonetaryValue` object equivalent to `monetaryValue` and whose `precision` may be equal to `precision`
 */
function unsafeMatchPrecision(monetaryValue, precision) {
    if (monetaryValue.precision >= precision) {
        return monetaryValue;
    }
    return {
        amount: monetaryValue.amount * pow10(precision - monetaryValue.precision),
        currency: monetaryValue.currency,
        precision: precision,
    };
}
/**
 * @public
 * Returns a monetary value whose precision is the provided one.
 * It is equivalent to the provided monetary value if the precision is equal or higher than the one of the provided monetary value.
 * Otherwise, it rounds the amount.
 * @remarks
 * The operation may cause overflows or imprecisions if the provided precision is big enough.
 * Also, note that the operation may be destructive if the precision is lower than the one of the provided monetary value.
 * See {@link unsafeSetPrecision} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value whose precision will be changed
 * @param precision - The precision that the monetary value shall be set to; must be a positive safe integer
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` object whose `precision` is equal to `precision` and that may be equivalent to `monetaryValue`
 */
function setPrecision(monetaryValue, precision, roundingFunction) {
    if (roundingFunction === void 0) { roundingFunction = roundHalfToEven; }
    assertValidMonetaryValue(monetaryValue);
    assertValidPrecision(precision);
    return unsafeSetPrecision(monetaryValue, precision, roundingFunction);
}
/**
 * @public
 * Returns a monetary value whose precision is the provided one.
 * It is equivalent to the provided monetary value if the precision is equal or higher than the one of the provided monetary value.
 * Otherwise, it rounds the amount.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions if the provided precision is big enough.
 * Also, note that the operation may be destructive if the precision is lower than the one of the provided monetary value.
 * See {@link setPrecision} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value whose precision will be changed
 * @param precision - The precision that the monetary value shall be set to
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` object whose `precision` is equal to `precision` and that may be equivalent to `monetaryValue`
 */
function unsafeSetPrecision(monetaryValue, precision, roundingFunction) {
    if (roundingFunction === void 0) { roundingFunction = roundHalfToEven; }
    if (monetaryValue.precision >= precision) {
        var divider = pow10(monetaryValue.precision - precision);
        var wholePart = monetaryValue.amount < 0
            ? Math.ceil(monetaryValue.amount / divider)
            : Math.floor(monetaryValue.amount / divider);
        var numerator = monetaryValue.amount % divider;
        return {
            amount: roundingFunction(wholePart, numerator, divider),
            currency: monetaryValue.currency,
            precision: precision,
        };
    }
    return {
        amount: monetaryValue.amount * pow10(precision - monetaryValue.precision),
        currency: monetaryValue.currency,
        precision: precision,
    };
}

/**
 * @public
 * Returns whether the two monetary values are equivalent.
 * @remarks
 * Contrary to most other binary operators, `equivalent` does not throw when the provided monetary values have different currencies.
 * Use {@link equal} if you want to determine the equality of monetary values that always have the same currency.
 * See {@link unsafeEquivalent} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` if the objects hold the same decimal value and have the same `currency`; `false` otherwise
 */
function equivalent(monetaryValue1, monetaryValue2) {
    assertValidMonetaryValue(monetaryValue1);
    assertValidMonetaryValue(monetaryValue2);
    return unsafeEquivalent(monetaryValue1, monetaryValue2);
}
/**
 * @public
 * Returns whether the two monetary values are equivalent.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link equivalent} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` if the objects hold the same decimal value and have the same `currency`; `false` otherwise
 */
function unsafeEquivalent(monetaryValue1, monetaryValue2) {
    if (monetaryValue1.currency !== monetaryValue2.currency) {
        return false;
    }
    var a = unsafeMatchPrecision(monetaryValue1, monetaryValue2.precision);
    var b = unsafeMatchPrecision(monetaryValue2, monetaryValue1.precision);
    return a.amount === b.amount;
}
/**
 * @public
 * Returns whether the two monetary values are identical.
 * @remarks
 * Contrary to most other binary operators, `identical` does not throw when the provided monetary values have different currencies.
 * See {@link unsafeIdentical} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` if the objects have the same `amount`, the same `currency` and the same `precision`; `false` otherwise
 */
function identical(monetaryValue1, monetaryValue2) {
    assertValidMonetaryValue(monetaryValue1);
    assertValidMonetaryValue(monetaryValue2);
    return unsafeIdentical(monetaryValue1, monetaryValue2);
}
/**
 * @public
 * Returns whether the two monetary values are identical.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link identical} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` if the objects have the same `amount`, the same `currency` and the same `precision`; `false` otherwise
 */
function unsafeIdentical(monetaryValue1, monetaryValue2) {
    return (monetaryValue1.currency === monetaryValue2.currency &&
        monetaryValue1.amount === monetaryValue2.amount &&
        monetaryValue1.precision === monetaryValue2.precision);
}
/**
 * @public
 * Returns a number whose sign represents whether the first monetary value is less than, equal to or greater than the second monetary value.
 * @remarks
 * See {@link unsafeCompare} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency as `monetaryValue1`
 * @returns respectively `-1`, `0` or `1` when `monetaryValue1` is less than, equal to or greater than `monetaryValue2`
 */
function compare(monetaryValue1, monetaryValue2) {
    assertValidMonetaryValue(monetaryValue1);
    assertValidMonetaryValue(monetaryValue2);
    assertSameCurrency(monetaryValue1, monetaryValue2);
    return unsafeCompare(monetaryValue1, monetaryValue2);
}
/**
 * @public
 * Returns a number whose sign represents whether the first monetary value is less than, equal to or greater than the second monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link compare} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns respectively `-1`, `0` or `1` when `monetaryValue1` is less than, equal to or greater than `monetaryValue2`
 */
function unsafeCompare(monetaryValue1, monetaryValue2) {
    var a = unsafeMatchPrecision(monetaryValue1, monetaryValue2.precision);
    var b = unsafeMatchPrecision(monetaryValue2, monetaryValue1.precision);
    if (a.amount < b.amount) {
        return -1 /* LT */;
    }
    if (a.amount > b.amount) {
        return 1 /* GT */;
    }
    return 0 /* EQ */;
}
/**
 * @public
 * Returns whether the first monetary value is strictly less than the second monetary value.
 * @remarks
 * See {@link unsafeLessThan} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency as `monetaryValue1`
 * @returns `true` when `monetaryValue1` holds a decimal value strictly less than `monetaryValue2`; `false` otherwise
 */
function lessThan(monetaryValue1, monetaryValue2) {
    assertValidMonetaryValue(monetaryValue1);
    assertValidMonetaryValue(monetaryValue2);
    assertSameCurrency(monetaryValue1, monetaryValue2);
    return unsafeLessThan(monetaryValue1, monetaryValue2);
}
/**
 * @public
 * Returns whether the first monetary value is strictly less than the second monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link lessThan} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` when `monetaryValue1` holds a decimal value strictly less than `monetaryValue2`; `false` otherwise
 */
function unsafeLessThan(monetaryValue1, monetaryValue2) {
    return unsafeCompare(monetaryValue1, monetaryValue2) === -1 /* LT */;
}
/**
 * @public
 * Returns whether the first monetary value is less than or equal to the second monetary value.
 * @remarks
 * See {@link unsafeLessThanOrEqual} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency as `monetaryValue1`
 * @returns `true` when `monetaryValue1` holds a decimal value less than or equivalent to `monetaryValue2`; `false` otherwise
 */
function lessThanOrEqual(monetaryValue1, monetaryValue2) {
    assertValidMonetaryValue(monetaryValue1);
    assertValidMonetaryValue(monetaryValue2);
    assertSameCurrency(monetaryValue1, monetaryValue2);
    return unsafeLessThanOrEqual(monetaryValue1, monetaryValue2);
}
/**
 * @public
 * Returns whether the first monetary value is less than or equal to the second monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link lessThanOrEqual} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` when `monetaryValue1` holds a decimal value less than or equivalent to `monetaryValue2`; `false` otherwise
 */
function unsafeLessThanOrEqual(monetaryValue1, monetaryValue2) {
    var comparison = unsafeCompare(monetaryValue1, monetaryValue2);
    return comparison === -1 /* LT */ || comparison === 0 /* EQ */;
}
/**
 * @public
 * Returns whether the two monetary values are equal.
 * @remarks
 * As with most of the other binary operators, `equal` throws when the provided monetary values have different currencies.
 * Use {@link equivalent} if you want to determine the equality of monetary values that may have a different currency.
 * See {@link unsafeEqual} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency as `monetaryValue1`
 * @returns `true` if the objects hold the same decimal value; `false` otherwise
 */
function equal(monetaryValue1, monetaryValue2) {
    assertValidMonetaryValue(monetaryValue1);
    assertValidMonetaryValue(monetaryValue2);
    assertSameCurrency(monetaryValue1, monetaryValue2);
    return unsafeEqual(monetaryValue1, monetaryValue2);
}
/**
 * @public
 * Returns whether the two monetary values are equal.
 * Does not check the validity of the arguments.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link equal} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` if the objects hold the same decimal value; `false` otherwise
 */
function unsafeEqual(monetaryValue1, monetaryValue2) {
    var a = unsafeMatchPrecision(monetaryValue1, monetaryValue2.precision);
    var b = unsafeMatchPrecision(monetaryValue2, monetaryValue1.precision);
    return a.amount === b.amount;
}
/**
 * @public
 * Returns whether the first monetary value is greater than or equal to the second monetary value.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link unsafeGreaterThanOrEqual} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency as `monetaryValue1`
 * @returns `true` when `monetaryValue1` holds a decimal value greater than or equivalent to `monetaryValue2`; `false` otherwise
 */
function greaterThanOrEqual(monetaryValue1, monetaryValue2) {
    assertValidMonetaryValue(monetaryValue1);
    assertValidMonetaryValue(monetaryValue2);
    assertSameCurrency(monetaryValue1, monetaryValue2);
    return unsafeGreaterThanOrEqual(monetaryValue1, monetaryValue2);
}
/**
 * @public
 * Returns whether the first monetary value is greater than or equal to the second monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link greaterThanOrEqual} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` when `monetaryValue1` holds a decimal value greater than or equivalent to `monetaryValue2`; `false` otherwise
 */
function unsafeGreaterThanOrEqual(monetaryValue1, monetaryValue2) {
    var comparison = unsafeCompare(monetaryValue1, monetaryValue2);
    return comparison === 1 /* GT */ || comparison === 0 /* EQ */;
}
/**
 * @public
 * Returns whether the first monetary value is strictly greater than the second monetary value.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link unsafeGreaterThan} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency as `monetaryValue1`
 * @returns `true` when `monetaryValue1` holds a decimal value greater than `monetaryValue2`; `false` otherwise
 */
function greaterThan(monetaryValue1, monetaryValue2) {
    assertValidMonetaryValue(monetaryValue1);
    assertValidMonetaryValue(monetaryValue2);
    assertSameCurrency(monetaryValue1, monetaryValue2);
    return unsafeGreaterThan(monetaryValue1, monetaryValue2);
}
/**
 * @public
 * Returns whether the first monetary value is strictly greater than the second monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * This comparison ignores the currencies.
 * See {@link greaterThan} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare the first with
 * @returns `true` when `monetaryValue1` holds a decimal value greater than `monetaryValue2`; `false` otherwise
 */
function unsafeGreaterThan(monetaryValue1, monetaryValue2) {
    return unsafeCompare(monetaryValue1, monetaryValue2) === 1 /* GT */;
}

/**
 * @public
 * Formats a monetary value into a string in the given locale.
 * @remarks
 * See {@link unsafeFormat} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value to format
 * @param locales - A string with a BCP 47 language tag, or an array of such strings.
 * @param options - An object to tweak the formatting
 * @returns a string that represents `monetaryValue` in the given `locale`
 */
function format(monetaryValue, locales, options) {
    if (options === void 0) { options = {}; }
    assertValidMonetaryValue(monetaryValue);
    return unsafeFormat(monetaryValue, locales, options);
}
/**
 * @public
 * Formats a monetary value into a string in the given locale.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link format} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value to format
 * @param locales - A string with a BCP 47 language tag, or an array of such strings
 * @param options - An object to tweak the formatting
 * @returns a string that represents `monetaryValue` in the given `locale`
 */
function unsafeFormat(monetaryValue, locales, formatOptions) {
    if (formatOptions === void 0) { formatOptions = {}; }
    var _a = formatOptions.signDisplay, signDisplay = _a === void 0 ? "auto" /* Auto */ : _a, _b = formatOptions.currencySign, currencySign = _b === void 0 ? "standard" /* Standard */ : _b;
    if (signDisplay === "auto" /* Auto */ &&
        currencySign === "standard" /* Standard */) {
        return nativeFormat(monetaryValue, locales, formatOptions);
    }
    return experimentalFormat(monetaryValue, locales, formatOptions);
}
function nativeFormat(monetaryValue, locales, formatOptions) {
    return Number.prototype.toLocaleString.call(unsafeToNumber(monetaryValue), locales, {
        localeMatcher: formatOptions.localeMatcher,
        currencyDisplay: formatOptions.currencyDisplay,
        useGrouping: formatOptions.useGrouping,
        style: 'currency',
        currency: monetaryValue.currency,
        minimumFractionDigits: monetaryValue.precision,
        maximumFractionDigits: monetaryValue.precision,
    });
}
function experimentalFormat(monetaryValue, locales, formatOptions) {
    var amount = monetaryValue.amount;
    var formattedMonetaryValue = nativeFormat(monetaryValue, locales, formatOptions);
    if (isSignRequired(formatOptions.signDisplay, amount)) {
        return putTheSign(formattedMonetaryValue, amount, formatOptions.currencySign);
    }
    return removeTheSign(formattedMonetaryValue, amount);
}
function putTheSign(formattedValue, amount, currencySign) {
    if (currencySign === void 0) { currencySign = "standard" /* Standard */; }
    if (amount >= 0) {
        return "+" + formattedValue;
    }
    if (currencySign === "accounting" /* Accounting */) {
        return "(" + formattedValue.replace('-', '') + ")";
    }
    return formattedValue;
}
function removeTheSign(formattedValue, amount) {
    if (amount >= 0) {
        return formattedValue;
    }
    return formattedValue.replace('-', '');
}
function isSignRequired(signDisplay, amount) {
    if (signDisplay === void 0) { signDisplay = "auto" /* Auto */; }
    return (signDisplay === "always" /* Always */ ||
        (signDisplay === "auto" /* Auto */ && amount < 0) ||
        (signDisplay === "except-zero" /* ExceptZero */ && amount !== 0));
}
/**
 * @public
 * Returns whether {@link format} and {@link unsafeFormat} are supported in your environment.
 * @remarks
 * Implementation taken from [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/toLocaleString#Checking_for_support_for_locales_and_options_arguments).
 */
function isFormatSupported() {
    return (typeof Intl === 'object' &&
        Intl !== null &&
        typeof Intl.NumberFormat === 'function');
}

/**
 * @public
 * Returns whether the provided argument is a monetary value.
 * Validates both the shape of the object and whether it is a valid monetary value.
 * Can be used as a type guard.
 * @remarks
 * Must be chained to another type guard if you want a more specific type for currency.
 * @param arg - Value to check
 * @returns `true` if `arg` is a of type `MonetaryValue` and has a valid `amount` and a valid `precision`; `false` otherwise
 */
function isMonetaryValue(arg) {
    var hasRightShape = typeof arg === 'object' &&
        arg !== null &&
        typeof arg.amount === 'number' &&
        typeof arg.currency === 'string' &&
        typeof arg.precision === 'number' &&
        arg.currency.length > 0;
    if (!hasRightShape) {
        return false;
    }
    var _a = arg, amount = _a.amount, precision = _a.precision;
    return (precision >= 0 &&
        precision <= 15 &&
        Number.isInteger(precision) &&
        Number.isSafeInteger(amount));
}
/**
 * @public
 * A convenience function to build a monetary value.
 * @remarks
 * See {@link unsafeCreate} for an implementation that does not validate the arguments.
 * @param amount - The amount that will be given to the monetary value; must be a safe integer
 * @param currency - The currency that will be given to the monetary value
 * @param precision - The precision that will be given to the monetary value; must a positive safe integer; defaults to `0`
 * @returns a `MonetaryValue` object whose `amount`, `currency` and `precision` are the ones provided
 */
function create(amount, currency, precision) {
    if (precision === void 0) { precision = 0; }
    var monetaryValue = unsafeCreate(amount, currency, precision);
    assertValidMonetaryValue(monetaryValue);
    return monetaryValue;
}
/**
 * @public
 * A convenience function to build a monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link create} for an implementation that validates the arguments.
 * @param amount - The amount that will be given to the monetary value
 * @param currency - The currency that will be given to the monetary value
 * @param precision - The precision that will be given to the monetary value; defaults to `0`
 * @returns a `MonetaryValue` object whose `amount`, `currency` and `precision` are the ones provided
 */
function unsafeCreate(amount, currency, precision) {
    if (precision === void 0) { precision = 0; }
    return { amount: amount, currency: currency, precision: precision };
}

/**
 * @public
 * Returns the monetary value with the greatest decimal amount.
 * @remarks
 * See {@link unsafeMaximum} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency
 * @returns `monetaryValue1` if its decimal value is greater than the one of `monetaryValue2` or, when they are equal, if its precision is lower than the one of `monetaryValue2`; otherwise, `monetaryValue2`
 */
function maximum(monetaryValue1, monetaryValue2) {
    assertValidMonetaryValue(monetaryValue1);
    assertValidMonetaryValue(monetaryValue2);
    assertSameCurrency(monetaryValue1, monetaryValue2);
    return unsafeMaximum(monetaryValue1, monetaryValue2);
}
/**
 * @public
 * Returns the monetary value with the greatest decimal amount.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link maximum} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`
 * @returns `monetaryValue1` if its decimal value is greater than the one of `monetaryValue2` or, when they are equal, if its precision is lower than the one of `monetaryValue2`; otherwise, `monetaryValue2`
 */
function unsafeMaximum(monetaryValue1, monetaryValue2) {
    var comparison = unsafeCompare(monetaryValue1, monetaryValue2);
    switch (comparison) {
        case 1 /* GT */:
            return monetaryValue1;
        case -1 /* LT */:
            return monetaryValue2;
        case 0 /* EQ */:
            return monetaryValue1.precision <= monetaryValue2.precision
                ? monetaryValue1
                : monetaryValue2;
        /* istanbul ignore next line */
        default:
            throw new TypeError('Unknown comparison value; this should not have happened');
    }
}
/**
 * @public
 * Returns the monetary value with the smallest decimal amount.
 * @remarks
 * See {@link unsafeMinimum} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`; must be of the same currency
 * @returns `monetaryValue2` if its decimal value is greater than the one of `monetaryValue1` or, when they are equal, if its precision is lower than the one of `monetaryValue1`; otherwise, `monetaryValue1`
 */
function minimum(monetaryValue1, monetaryValue2) {
    assertValidMonetaryValue(monetaryValue1);
    assertValidMonetaryValue(monetaryValue2);
    assertSameCurrency(monetaryValue1, monetaryValue2);
    return unsafeMinimum(monetaryValue1, monetaryValue2);
}
/**
 * @public
 * Returns the monetary value with the smallest decimal amount.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link minimum} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to compare to `monetaryValue1`
 * @returns `monetaryValue2` if its decimal value is greater than the one of `monetaryValue1` or, when they are equal, if its precision is lower than the one of `monetaryValue1`; otherwise, `monetaryValue1`
 */
function unsafeMinimum(monetaryValue1, monetaryValue2) {
    var comparison = unsafeCompare(monetaryValue1, monetaryValue2);
    switch (comparison) {
        case 1 /* GT */:
            return monetaryValue2;
        case -1 /* LT */:
            return monetaryValue1;
        case 0 /* EQ */:
            return monetaryValue1.precision >= monetaryValue2.precision
                ? monetaryValue1
                : monetaryValue2;
        /* istanbul ignore next line */
        default:
            throw new TypeError('Unknown comparison value; this should not have happened');
    }
}
/**
 * @public
 * Returns the sum of the two monetary values.
 * If they have a different `precision`, the highest is used.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link unsafeAdd} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to add to `monetaryValue1`; must be of the same currency
 * @returns a `MonetaryValue` object whose decimal value is the sum of the ones of `monetaryValue1` and `monetaryValue2`, with the same `currency` as `monetaryValue1` and `monetaryValue2`
 */
function add(monetaryValue1, monetaryValue2) {
    assertValidMonetaryValue(monetaryValue1);
    assertValidMonetaryValue(monetaryValue2);
    assertSameCurrency(monetaryValue1, monetaryValue2);
    return unsafeAdd(monetaryValue1, monetaryValue2);
}
/**
 * @public
 * Returns the sum of the two monetary values.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link add} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to add to `monetaryValue1`
 * @returns a `MonetaryValue` object whose decimal value is the sum of the ones of `monetaryValue1` and `monetaryValue2`, with the same `currency` as `monetaryValue1` and `monetaryValue2`
 */
function unsafeAdd(monetaryValue1, monetaryValue2) {
    var a = unsafeMatchPrecision(monetaryValue1, monetaryValue2.precision);
    var b = unsafeMatchPrecision(monetaryValue2, monetaryValue1.precision);
    return {
        amount: a.amount + b.amount,
        currency: a.currency,
        precision: a.precision,
    };
}
/**
 * @public
 * Returns the difference of the two monetary values.
 * If they have a different `precision`, the highest is used.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link unsafeSubtract} for an implementation that does not validate the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to subtract from `monetaryValue1`; must be of the same currency
 * @returns a `MonetaryValue` whose decimal value is the difference between the ones of `monetaryValue1` and `monetaryValue2`, with the same `currency` as `monetaryValue1` and `monetaryValue2`
 */
function subtract(monetaryValue1, monetaryValue2) {
    assertValidMonetaryValue(monetaryValue1);
    assertValidMonetaryValue(monetaryValue2);
    assertSameCurrency(monetaryValue1, monetaryValue2);
    return unsafeSubtract(monetaryValue1, monetaryValue2);
}
/**
 * @public
 * Returns the difference of the two monetary values.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link subtract} for an implementation that validates the arguments.
 * @param monetaryValue1 - A first monetary value
 * @param monetaryValue2 - A second monetary value to subtract to the first
 */
function unsafeSubtract(monetaryValue1, monetaryValue2) {
    var a = unsafeMatchPrecision(monetaryValue1, monetaryValue2.precision);
    var b = unsafeMatchPrecision(monetaryValue2, monetaryValue1.precision);
    return {
        amount: a.amount - b.amount,
        currency: a.currency,
        precision: a.precision,
    };
}

/**
 * @public
 * Returns a monetary value whose amount is the product of the monetary value's amount and the decimal factor.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link unsafeMultiply} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value to multiply
 * @param factor - The coefficient of the number that `monetaryValue` will be multiplied by; must be a safe integer
 * @param factorPrecision - The precision of the number that `monetaryValue` will be multiplied by
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half-to-even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` whose decimal value is the product of the ones of `monetaryValue` and `factor / (10 ** factorPrecision)`, with the same `currency` as `monetaryValue`
 */
function multiply(monetaryValue, factor, factorPrecision, roundingFunction) {
    if (roundingFunction === void 0) { roundingFunction = roundHalfToEven; }
    assertValidMonetaryValue(monetaryValue);
    assertSafeInteger(factor);
    assertValidPrecision(factorPrecision);
    return unsafeMultiply(monetaryValue, factor, factorPrecision, roundingFunction);
}
/**
 * @public
 * Returns a monetary value whose amount is the product of the monetary value's amount and the decimal factor.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link multiply} for an implementation that validates the arguments.
 * @param factor - The coefficient of the number that `monetaryValue` will be multiplied by
 * @param factorPrecision - The precision of the number that `monetaryValue` will be multiplied by
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half-to-even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` whose decimal value is the product of the ones of `monetaryValue` and `factor / (10 ** factorPrecision)`, with the same `currency` as `monetaryValue`
 */
function unsafeMultiply(monetaryValue, factor, factorPrecision, roundingFunction) {
    if (roundingFunction === void 0) { roundingFunction = roundHalfToEven; }
    return unsafeIntegerDivide({
        amount: monetaryValue.amount * factor,
        currency: monetaryValue.currency,
        precision: monetaryValue.precision,
    }, pow10(factorPrecision), roundingFunction);
}
/**
 * @public
 * Returns a monetary value whose amount is the quotient of the monetary value's amount and the divider.
 * @remarks
 * The operation may cause overflows or imprecisions depending on the provided rounding function.
 * See {@link unsafeIntegerDivide} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value to divide
 * @param divider - The number that `monetaryValue` will be divided by; must be a safe integer different than `0`
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` whose decimal value is the whole quotient of the ones of `monetaryValue` and `divider`, with the same `currency` as `monetaryValue`
 */
function integerDivide(monetaryValue, divider, roundingFunction) {
    if (roundingFunction === void 0) { roundingFunction = roundHalfToEven; }
    assertValidMonetaryValue(monetaryValue);
    assertNonZeroSafeInteger(divider);
    return unsafeIntegerDivide(monetaryValue, divider, roundingFunction);
}
/**
 * @public
 * Returns a monetary value whose amount is the quotient of the monetary value's amount and the divider.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions depending on the provided rounding function.
 * See {@link integerDivide} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value to divide
 * @param divider - The number that `monetaryValue` will be divided by
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` whose decimal value is the whole quotient of the ones of `monetaryValue` and `divider`, with the same `currency` as `monetaryValue`
 */
function unsafeIntegerDivide(monetaryValue, divider, roundingFunction) {
    if (roundingFunction === void 0) { roundingFunction = roundHalfToEven; }
    var wholePart = Math.trunc(monetaryValue.amount / divider);
    var numerator = monetaryValue.amount % divider;
    return {
        amount: roundingFunction(wholePart, numerator, divider),
        currency: monetaryValue.currency,
        precision: monetaryValue.precision,
    };
}
/**
 * @public
 * Returns a monetary value whose amount is the division of the provided monetary value's amount and the decimal divider.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link unsafeDivide} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value to divide
 * @param divider - The coefficient of the number that `monetaryValue` will be divided by; must be a safe integer different than `0`
 * @param dividerPrecision - The precision of the number that `monetaryValue` will be divided by
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` whose decimal value is the division of the ones of `monetaryValue` and `divider / (10 ** dividerPrecision)`, with the same `currency` as `monetaryValue`
 */
function divide(monetaryValue, divider, dividerPrecision, roundingFunction) {
    if (roundingFunction === void 0) { roundingFunction = roundHalfToEven; }
    assertValidMonetaryValue(monetaryValue);
    assertNonZeroSafeInteger(divider);
    assertValidPrecision(dividerPrecision);
    return unsafeDivide(monetaryValue, divider, dividerPrecision, roundingFunction);
}
/**
 * @public
 * Returns a monetary value whose amount is the division of the provided monetary value's amount and the decimal divider.
 * Does not check the validity of the arguments.
 * @remarks
 * The operation may cause overflows or imprecisions.
 * See {@link divide} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value to divide
 * @param divider - The coefficient of the number that `monetaryValue` will be divided by
 * @param dividerPrecision - The precision of the number that `monetaryValue` will be divided by
 * @param roundingFunction - The rounding method to apply to the result's amount (by default {@link roundHalfToEven | round half to even}); see {@link RoundingFunction}
 * @returns a `MonetaryValue` whose decimal value is the division of the ones of `monetaryValue` and `divider / (10 ** dividerPrecision)`, with the same `currency` as `monetaryValue`
 */
function unsafeDivide(monetaryValue, divider, dividerPrecision, roundingFunction) {
    if (roundingFunction === void 0) { roundingFunction = roundHalfToEven; }
    return unsafeIntegerDivide({
        amount: monetaryValue.amount * pow10(dividerPrecision),
        currency: monetaryValue.currency,
        precision: monetaryValue.precision,
    }, divider, roundingFunction);
}
/**
 * @public
 * Splits a monetary value in shares of given weights, preserving the precision.
 * Useful when you want to perform successive divisions but must ensure that the sum of the quotients is equal to the provided monetary value.
 * Does not check the validity of the arguments.
 * @remarks
 * A 1-dimension Brensenham algorithm is used to handle rounding.
 * When the division of the amount by a given weight results to a non-integer number, the decimal part is reported to the next share.
 * This ensures, in a single pass over the weights, that the sum of the shares is equal to the provided monetary value.
 * It is however unpredictable as the order of the weights will change the result.
 * You may need to a implement a similar function yourself if you need control over the rounding.
 * The operation may cause of overflows or imprecisions if negative weights are provided.
 * See {@link unsafeAllocate} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value that will be split
 * @param weights - An array of numbers, positive or negative, that will determine the allocation; their sum must be different than `0`
 */
function allocate(monetaryValue, weights) {
    assertValidMonetaryValue(monetaryValue);
    var total = sum(weights);
    assertStrictlyPositiveSafeInteger(total);
    return internalAllocate(monetaryValue, weights, total);
}
/**
 * @public
 * Splits a monetary value in shares of given weights, preserving the precision.
 * Useful when you want to perform successive divisions but must ensure that the sum of the quotients is equal to the provided monetary value.
 * @remarks
 * A 1-dimension Brensenham algorithm is used to handle rounding.
 * When the division of the amount by a given weight results to a non-integer number, the decimal part is reported to the next share.
 * This ensures, in a single pass over the weights, that the sum of the shares is equal to the provided monetary value.
 * It is however unpredictable as the order of the weights will change the result.
 * You may need to a implement a similar function yourself if you need control over the rounding.
 * The operation may cause of overflows or imprecisions if negative weights are provided.
 * See {@link allocate} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value that will be split
 * @param weights - An array of numbers, positive or negative, that will determine the allocation
 */
function unsafeAllocate(monetaryValue, weights) {
    return internalAllocate(monetaryValue, weights, sum(weights));
}
function internalAllocate(monetaryValue, weights, total) {
    var amount = monetaryValue.amount, currency = monetaryValue.currency, precision = monetaryValue.precision;
    var shares = allocate$1(amount, weights, total);
    return shares.map(function (share) { return ({
        amount: share,
        currency: currency,
        precision: precision,
    }); });
}

/**
 * @public
 * Returns the provided monetary value with an amount of the opposite sign.
 * @remarks
 * See {@link unsafeNegate} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value whose sign will be changed
 * @returns a `MonetaryValue` object with the same `currency` and `precision` as `monetaryValue` but whose `amount` is `-monetaryValue.amount`
 */
function negate(monetaryValue) {
    assertValidMonetaryValue(monetaryValue);
    return unsafeNegate(monetaryValue);
}
/**
 * @public
 * Returns the provided monetary value with an amount of the opposite sign.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link negate} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value whose sign will be changed
 * @returns a `MonetaryValue` object with the same `currency` and `precision` as `monetaryValue` but whose `amount` is `-monetaryValue.amount`
 */
function unsafeNegate(monetaryValue) {
    return {
        amount: -monetaryValue.amount,
        currency: monetaryValue.currency,
        precision: monetaryValue.precision,
    };
}
/**
 * @public
 * Returns the provided monetary value with an amount that is positive.
 * @remarks
 * See {@link unsafeAbsolute} for an implementation that does not validate the arguments.
 * @param monetaryValue - The monetary value whose sign will be removed
 * @returns a `MonetaryValue` object with the same `currency` and `precision` as `monetaryValue` but whose `amount` is `abs(monetaryValue.amount)`
 */
function absolute(monetaryValue) {
    assertValidMonetaryValue(monetaryValue);
    return unsafeAbsolute(monetaryValue);
}
/**
 * @public
 * Returns the provided monetary value with an amount that is positive.
 * Does not check the validity of the arguments.
 * @remarks
 * See {@link absolute} for an implementation that validates the arguments.
 * @param monetaryValue - The monetary value whose sign will be removed
 * @returns a `MonetaryValue` object with the same `currency` and `precision` as `monetaryValue` but whose `amount` is `abs(monetaryValue.amount)`
 */
function unsafeAbsolute(monetaryValue) {
    return {
        amount: abs(monetaryValue.amount),
        currency: monetaryValue.currency,
        precision: monetaryValue.precision,
    };
}

export { absolute, add, allocate, compare, create, divide, equal, equivalent, format, fromNumber, fromString, greaterThan, greaterThanOrEqual, identical, integerDivide, isFormatSupported, isMonetaryValue, lessThan, lessThanOrEqual, matchPrecision, maximum, minimum, multiply, negate, roundAwayFromZero, roundDown, roundHalfAwayFromZero, roundHalfDown, roundHalfToEven, roundHalfToOdd, roundHalfTowardsZero, roundHalfUp, roundTowardsZero, roundUp, setPrecision, subtract, toNumber, toString, unsafeAbsolute, unsafeAdd, unsafeAllocate, unsafeCompare, unsafeCreate, unsafeDivide, unsafeEqual, unsafeEquivalent, unsafeFormat, unsafeFromNumber, unsafeFromString, unsafeGreaterThan, unsafeGreaterThanOrEqual, unsafeIdentical, unsafeIntegerDivide, unsafeLessThan, unsafeLessThanOrEqual, unsafeMatchPrecision, unsafeMaximum, unsafeMinimum, unsafeMultiply, unsafeNegate, unsafeSetPrecision, unsafeSubtract, unsafeToNumber, unsafeToString };
